@@ -16,8 +16,8 @@
 */
 #include <string.h>
 #include <stdlib.h>
-#include <endian.h>
 #include <stdio.h> // For debugging
+#include "endian.h"
 #include "amfparser.h"
 
 char amf_comparestrings_c(struct amfstring* a, const char* b)
@@ -71,7 +71,7 @@ struct amf* amf_parse(const unsigned char* buf, int len)
       obj=&amf->items[amf->itemcount-1].object;
       // TODO: recurse into unfinished member-objects (unimportant, I haven't seen any objects within objects so far)
       // Add member and set name
-      i=be16toh(*(short*)buf);
+      i=be16(*(short*)buf);
       buf=&buf[sizeof(short)];
       if(&buf[i]>=end){printf("Warning: skipping object item with name exceeding RTMP size (0x%x)\n", i);}
       if(&buf[i]<end && buf[i]!=9) // 9=end-of-object
@@ -115,7 +115,7 @@ struct amf* amf_parse(const unsigned char* buf, int len)
         item=&obj->members[obj->membercount-1].value;
       else
         item=amf_newitem(amf);
-      i=be16toh(*(short*)buf);
+      i=be16(*(short*)buf);
       buf=&buf[sizeof(short)];
       item->type=AMF_STRING;
       item->string.length=i;
