@@ -5,6 +5,7 @@ if [ "$host" = "" ]; then
   if [ "$host" = "" ]; then host='i386-pc-linux-gnu'; fi
   echo "No target host specified (argv[1]), defaulting to ${host}"
 fi
+./configure --host="$host"
 here="`pwd`"
 if [ ! -e curlprefix ]; then
   wget -c http://curl.haxx.se/download/curl-7.40.0.tar.bz2
@@ -17,14 +18,19 @@ if [ ! -e curlprefix ]; then
   make install
   cd "$here"
 fi
-export PATH="${here}/curlprefix/bin:${PATH}"
-# Select a crosscompiler (if we can find one)
-if which "${host}-gcc" > /dev/null 2> /dev/null && [ "`which "${host}-gcc" 2> /dev/null`" != "" ]; then
-  export CC="${host}-gcc"
-elif which "${host}-clang" > /dev/null 2> /dev/null && [ "`which "${host}-clang" 2> /dev/null`" != "" ]; then
-  export CC="${host}-clang"
-elif which "${host}-cc" > /dev/null 2> /dev/null && [ "`which "${host}-cc" 2> /dev/null`" != "" ]; then
-  export CC="${host}-cc"
+if grep -q 'LIBS+=-liconv' config.mk && [ ! -e iconvprefix ]; then
+  wget -c http://ftp.gnu.org/gnu/libiconv/libiconv-1.14.tar.gz
+  tar -xzf libiconv-1.14.tar.gz
+  cd libiconv-1.14
+  mkdir -p build
+  cd build
+  ../configure --prefix="${here}/iconvprefix" --host="`echo "$host" | sed -e 's/android/gnu/'`" --enable-static --disable-shared CC="${host}-gcc" # libiconv does not handle android well, so we pretend it's GNU and specify the compiler
+  make
+  make install
+  cd "$here"
 fi
+export PATH="${here}/curlprefix/bin:${PATH}"
+echo "CFLAGS+=-I${here}/iconvprefix/include" >> config.mk
+echo "LDFLAGS+=-L${here}/iconvprefix/lib" >> config.mk
 make
 make utils
